import Dexie from 'dexie'
import { Map } from '@/application/types'

export class MapStorage {
  private dexie: Dexie

  constructor() {
    this.dexie = new Dexie('maps')
    this.dexie.version(1).stores({
      maps: 'id, name, createdAt, updatedAt'
    })
  }

  async add(map: Map) {
    try {
      await this.dexie.table('maps').put({
        ...map
      })
    } catch (error) {
      console.error(`Failed to add map ${map.id}:`, error)
    }
  }

  async get(id: string): Promise<Map | null> {
    try {
      const map = await this.dexie.table('maps').get(id)
      return map || null
    } catch (error) {
      console.error(`Failed to retrieve map ${id}:`, error)
      return null
    }
  }

  async getAll(): Promise<Map[]> {
    try {
      return await this.dexie.table('maps').toArray()
    } catch (error) {
      console.error('Failed to retrieve all maps:', error)
      return []
    }
  }

  async getByName(name: string): Promise<Map[]> {
    try {
      return await this.dexie.table('maps').where('name').equals(name).toArray()
    } catch (error) {
      console.error(`Failed to retrieve maps with name ${name}:`, error)
      return []
    }
  }

  async delete(id: string) {
    try {
      await this.dexie.table('maps').delete(id)
    } catch (error) {
      console.error(`Failed to delete map ${id}:`, error)
    }
  }

  async clear() {
    try {
      await this.dexie.table('maps').clear()
    } catch (error) {
      console.error('Failed to clear maps storage:', error)
    }
  }

  async update(id: string, updates: Partial<Map>) {
    try {
      const map = await this.get(id)
      if (map) {
        const updatedMap = {
          ...map,
          ...updates
        }
        await this.add(updatedMap)
        return true
      }
      return false
    } catch (error) {
      console.error(`Failed to update map ${id}:`, error)
      return false
    }
  }
}